package me.sheimi.sgit.activities.explorer;

import java.io.File;
import java.io.FileFilter;

import me.sheimi.sgit.R;
import me.sheimi.sgit.database.RepoContract;
import me.sheimi.sgit.database.RepoDbManager;
import me.sheimi.sgit.database.models.Repo;
import me.sheimi.sgit.repo.tasks.repo.InitLocalTask;
import android.app.Activity;
import android.app.AlertDialog;
import android.content.ContentValues;
import android.content.DialogInterface;
import android.content.DialogInterface.OnClickListener;
import android.content.Intent;
import android.os.Environment;
import android.view.Menu;
import android.view.MenuItem;
import android.view.View;
import android.widget.AdapterView;

public class ImportRepositoryActivity extends FileExplorerActivity {

    @Override
    protected File getRootFolder() {
        return Environment.getExternalStorageDirectory();
    }

    @Override
    protected FileFilter getExplorerFileFilter() {
        return new FileFilter() {
            @Override
            public boolean accept(File file) {
                if (!file.isDirectory())
                    return false;
                return true;
            }
        };
    }

    @Override
    public boolean onCreateOptionsMenu(Menu menu) {
        // Inflate the menu; this adds items to the action bar if it is present.
        getMenuInflater().inflate(R.menu.import_repo, menu);
        return true;
    }

    @Override
    public boolean onOptionsItemSelected(MenuItem item) {
        switch (item.getItemId()) {
            case R.id.action_create_external:
                File dotGit = new File(getCurrentDir(), Repo.DOT_GIT_DIR);
                if (dotGit.exists()) {
                    showToastMessage(R.string.alert_is_already_a_git_repo);
                    return true;
                }
                showMessageDialog(R.string.dialog_create_external_title,
                        R.string.dialog_create_external_msg,
                        R.string.dialog_create_external_positive_label,
                        new OnClickListener() {

                            @Override
                            public void onClick(DialogInterface dialog,
                                    int which) {
                                createExternalGitRepo();
                            }
                        });
                return true;
        }
        return super.onOptionsItemSelected(item);
    }

    @Override
    protected AdapterView.OnItemClickListener getOnListItemClickListener() {
        return new AdapterView.OnItemClickListener() {
            @Override
            public void onItemClick(AdapterView<?> adapterView, View view,
                    int position, long id) {
                final File file = mFilesListAdapter.getItem(position);
                File dotGit = new File(file, Repo.DOT_GIT_DIR);
                if (!dotGit.exists()) {
                    setCurrentDir(file);
                    return;
                }
                AlertDialog.Builder builder = new AlertDialog.Builder(
                        ImportRepositoryActivity.this);
                builder.setTitle(R.string.dialog_comfirm_import_repo_title);
                builder.setMessage(R.string.dialog_comfirm_import_repo_msg);
                builder.setNegativeButton(R.string.label_cancel,
                        new DialogInterface.OnClickListener() {
                            @Override
                            public void onClick(
                                    DialogInterface dialogInterface, int i) {
                                setCurrentDir(file);
                            }
                        });
                builder.setPositiveButton(R.string.label_import,
                        new DialogInterface.OnClickListener() {
                            @Override
                            public void onClick(
                                    DialogInterface dialogInterface, int i) {
                                Intent intent = new Intent();
                                intent.putExtra(RESULT_PATH,
                                        file.getAbsolutePath());
                                setResult(Activity.RESULT_OK, intent);
                                finish();
                            }
                        });
                builder.show();
            }
        };
    }

    @Override
    protected AdapterView.OnItemLongClickListener getOnListItemLongClickListener() {
        return null;
    }

    private void createExternalGitRepo() {
        File current = getCurrentDir();
        String local = Repo.EXTERNAL_PREFIX + current;
        ContentValues values = new ContentValues();
        values.put(RepoContract.RepoEntry.COLUMN_NAME_LOCAL_PATH, local);
        values.put(RepoContract.RepoEntry.COLUMN_NAME_REMOTE_URL,
                "local repository");
        values.put(RepoContract.RepoEntry.COLUMN_NAME_REPO_STATUS,
                RepoContract.REPO_STATUS_NULL);
        values.put(RepoContract.RepoEntry.COLUMN_NAME_USERNAME, "");
        values.put(RepoContract.RepoEntry.COLUMN_NAME_PASSWORD, "");
        long id = RepoDbManager.insertRepo(values);
        Repo repo = Repo.getRepoById(this, id);

        InitLocalTask task = new InitLocalTask(repo);
        task.executeTask();
        finish();
    }
}
